package com.qihoo360.apullsdk.apull.view.impl

import android.app.Activity
import android.content.Context
import android.media.MediaMetadataRetriever
import android.view.GestureDetector
import android.view.MotionEvent
import android.view.View
import android.view.animation.Animation
import android.view.animation.RotateAnimation
import com.qihoo360.apullsdk.R
import com.qihoo360.apullsdk.apull.ApullConst
import com.qihoo360.apullsdk.apull.control.sync.DownloadSatusManager
import com.qihoo360.apullsdk.apull.protocol.model.ApullTemplateBase
import com.qihoo360.apullsdk.apull.protocol.model.impl.TemplateApullMv
import com.qihoo360.apullsdk.apull.protocol.model.impl.mv.ApullMvItem
import com.qihoo360.apullsdk.export.ApullRewardVideoInterface
import com.qihoo360.apullsdk.export.ApullRewardVideoInterface.Companion.template2Json
import com.qihoo360.apullsdk.support.imageconfig.ImageDownloaderConfig
import com.qihoo360.apullsdk.support.imageconfig.ImageLoaderWrapper
import com.qihoo360.apullsdk.utils.tryCatch
import com.qihoo360.mobilesafe.support.log.Logger
import kotlinx.android.synthetic.main.apullsdk_container_apull_reward_video_end.view.*
import java.util.*

val  TAG = "ContainerApullMvVideoEnd"
class ContainerApullMv605VideoEnd(context: Context?, template: ApullTemplateBase?, apullRewardVideoInterface: ApullRewardVideoInterface) : ContainerApullMvBase(context, template) {

    private var mApullRewardVideoInterface : ApullRewardVideoInterface = apullRewardVideoInterface

    override fun initView(template: ApullTemplateBase?) {
        View.inflate(context, R.layout.apullsdk_container_apull_reward_video_end,this)
        DownloadSatusManager.register(this)
    }

    override fun updateView(template: ApullTemplateBase?) {
      template?.apply {
          if(template is TemplateApullMv && mTemplateApullMv != template){
              visibility = View.VISIBLE
              mTemplateApullMv = template
              apullMvItem = mTemplateApullMv.defaultItem
              updateText()
              updateImage()
              addClickListener()
          }
      }
    }

    private fun addClickListener() {
        mBtn?.setOnTouchListener { _, event -> btnDector.onTouchEvent(event) }
        mClose?.setOnClickListener {
            if(context is Activity){
                val activity = context as? Activity
                if(activity?.isFinishing == false){
                    activity.finish()
                    activity.overridePendingTransition(0,0)
                }
            }
        }
    }

     private fun startAnimation() {
         val rotateAnim = RotateAnimation(-5f, 5f, Animation.RELATIVE_TO_SELF, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f)
         rotateAnim.duration = 100
         mBtn.startAnimation(rotateAnim)
    }

    private fun updateImage() {
        tryCatch(catchBlock = {
           Logger.handleException(it)
        }) {
            if(apullMvItem?.adm_type == 3){
                setVideoBg(apullMvItem.adm?._native?.video?.url)
                mAppIcon?.let {
                    ImageLoaderWrapper.getInstance().displayImage(apullMvItem?.adm?._native?.logo,
                            it, ImageDownloaderConfig.getRoundedIconOptions(context))
                }
            }
        }
    }

    private fun setVideoBg(url: String?) {
        Thread({
            if(!url.isNullOrEmpty()){
                val mmr = MediaMetadataRetriever()
                tryCatch(finalBlock = {
                    mmr.release()//释放资源
                },catchBlock = {
                    Logger.handleException(it)
                }){
                    mmr.setDataSource(url, HashMap())
                    val bitmap = mmr.frameAtTime//获取第一帧图片
                    if(bitmap != null) {
                        mHandler?.post {
                            mVideoBgImg?.setImageBitmap(bitmap)
                            mVideoBgImg?.visibility = View.VISIBLE
                        }
                    }
                }
            }
        },"camv").start()

    }

    private fun updateText() {
        if(apullMvItem?.interaction_type == 2){
            mBtn?.text = context.getString(R.string.apullsdk_lookinfo)
        }else if(apullMvItem?.interaction_type == 3){
            if(apullMvItem?.status == ApullConst.STATUS_TYPE_DOWNLOADING
                    ||apullMvItem?.status == ApullConst.STATUS_TYPE_UPDATE_PROGRESS ){
                mBtn?.text = context.getString(R.string.apullsdk_webview_app_start_download_tips)
            }else if(apullMvItem?.status == ApullConst.STATUS_TYPE_DOWNLOAD_FINISHED
                    || apullMvItem?.status == ApullConst.STATUS_TYPE_INSTALL){
                mBtn?.text = context.getString(R.string.apullsdk_app_download_install)
            }else if(apullMvItem?.status == ApullConst.STATUS_TYPE_INSTALL_FINISHED){
                mBtn?.text = context.getString(R.string.apullsdk_app_open)
            }else{
                mBtn?.text = context.getString(R.string.apullsdk_app_download_immediately)
            }
        }

        mAppDesc?.text = apullMvItem?.adm?._native?.desc

        if(apullMvItem?.appNameOrTitle.isNullOrEmpty()){
            mAppTitle?.visibility = View.GONE
        }else{
            mAppTitle?.text = apullMvItem?.appNameOrTitle
            mAppTitle?.visibility = View.VISIBLE
        }
    }

    override fun onThemeChanged() {

    }

    override fun updateDownloadStatus() {
        mHandler.post{
            if(apullMvItem?.status == ApullConst.STATUS_TYPE_INSTALL_FINISHED){
                if(mTemplateApullMv?.installed_reported == false){
                    mApullRewardVideoInterface.onApkInstalled(template2Json(mTemplateApullMv,this@ContainerApullMv605VideoEnd.context))
                }
                handleAppInstalled()
            }
            updateText()
        }
    }

    override fun onImageEnableChange(enable: Boolean) {
        updateImage()
    }

    override fun getTemplate(): ApullTemplateBase? {
        return  mTemplateApullMv
    }

    private var btnGestureListener = object : GestureDetector.OnGestureListener{
        var clickParams = ApullMvItem.GdtClickParams()
        var containerLocation: IntArray = IntArray(2)
        override fun onShowPress(e: MotionEvent?) {
        }

        override fun onSingleTapUp(e: MotionEvent?): Boolean {
            Logger.d(TAG, "btnGestureListener onSingleTapUp x ==" + e?.x)
            clickParams.viewW = this@ContainerApullMv605VideoEnd.width
            clickParams.viewH = this@ContainerApullMv605VideoEnd.height
            this@ContainerApullMv605VideoEnd.getLocationOnScreen(containerLocation)
            if (e != null) {
                clickParams.upX = e.rawX.toInt() - containerLocation[0]
                clickParams.upY = e.rawY.toInt() - containerLocation[1]
            }
            startAnimation()
            if(mTemplateApullMv?.click_reported == false){
                mApullRewardVideoInterface.onClick2Download(template2Json(mTemplateApullMv,this@ContainerApullMv605VideoEnd.context))
            }
            if(apullMvItem?.interaction_type == 3 &&
                    (apullMvItem?.status == ApullConst.STATUS_TYPE_DOWNLOADING || apullMvItem?.status == ApullConst.STATUS_TYPE_UPDATE_PROGRESS )){
                return true
            }
            handleDownLoadBtnClick(clickParams)
            return true
        }

        override fun onDown(e: MotionEvent?): Boolean {
            Logger.d(TAG, "btnGestureListener onDown x ==" + e?.x)
            this@ContainerApullMv605VideoEnd.getLocationOnScreen(containerLocation)
            if (e != null) {
                clickParams.downX = e.rawX.toInt() - containerLocation[0]
                clickParams.downY = e.rawY.toInt() - containerLocation[1]
            }
            return true
        }

        override fun onFling(e1: MotionEvent?, e2: MotionEvent?, velocityX: Float, velocityY: Float): Boolean {
            return false
        }

        override fun onScroll(e1: MotionEvent?, e2: MotionEvent?, distanceX: Float, distanceY: Float): Boolean {
            return false
        }

        override fun onLongPress(e: MotionEvent?) {
        }
    }

    private var btnDector = GestureDetector (context, btnGestureListener)
}