package com.qihoo360.apullsdk.export

import android.content.Context
import com.qihoo360.apullsdk.apull.ApullConst
import com.qihoo360.apullsdk.apull.protocol.model.ApullTemplateBase
import com.qihoo360.apullsdk.apull.protocol.model.impl.TemplateGdtSdkRewardVideo
import com.qihoo360.apullsdk.apull.protocol.model.impl.TemplateTouTiaoSdkRewardVideo
import com.qihoo360.apullsdk.apull.protocol.model.impl.mv.ApullMvItem
import com.qihoo360.apullsdk.apull.protocol.model.impl.toutiao.ApullTouTiaoItem
import com.qihoo360.apullsdk.utils.PackageUtil
import com.qihoo360.apullsdk.utils.tryCatch
import com.qihoo360.mobilesafe.support.log.Logger
import org.json.JSONObject


interface ApullRewardVideoInterface {
    /**
     * 激励视频播放完成
     */
    fun onVideoPlayComplete(json: String?) = Unit

    /**
     * 加载视频模板失败或播放失败等
     */
    fun onRewardVideoFail(errorCode: Int)  = Unit

    /**
     * 点击下载
     */
    fun onClick2Download(json: String?) = Unit

    /**
     * 应用在安装完成
     */
    fun onApkInstalled(json: String?) = Unit


    //TT Gdt sdk 激励视频
    /**
     * 广告加载成功
     **/
    fun onAdLoad() = Unit

    /**
     * 视频素材缓存成功
     */
    fun onVideoCached() = Unit

    /**
     * 激励视频广告页面展示
     */
    fun  onADShow() = Unit

    /**
     * 激励视频广告曝光
     */
    fun onADExpose() = Unit

    /**
     * 激励视频广告被点击
     */
    fun  onADClick() = Unit

    /**
     * 激励视频触发激励（观看视频大于一定时长或者视频播放完毕）
     */
    fun onReward() = Unit

    /**
     * 激励视频播放完毕
     */
    fun onVideoComplete() = Unit

    /**
     * 激励视频广告被关闭
     */
    fun onADClose() = Unit

    /**
     * 广告流程出错
     */
    fun onError() = Unit

    //end TT  gdt sdk 激励视频

    //start tt sdk 激励视频,提供的额外回调

    fun onDownloadActive(totalBytes:Long,currentBytes:Long,fileName:String?,appName:String?) = Unit

    fun onDownloadPaused(totalBytes:Long,currentBytes:Long,fileName:String,appName:String?) = Unit

    fun onDownloadFailed(totalBytes:Long,currentBytes:Long,fileName:String,appName:String?) = Unit

    fun onDownloadFinished(totalBytes:Long,fileName:String,appName:String?) = Unit

    fun onInstalled(fileName: String,appName: String?) = Unit

    //end tt sdk 激励视频

    companion object {
        //目前只支持头条和mv
        fun template2Json(template: ApullTemplateBase?,context:Context?): String {
            val jsonObject = JSONObject()

            if (template != null) {
                tryCatch{
                    jsonObject.put("scene", template.scene)
                    jsonObject.put("subScene", template.subscene)

                    var mOpenType = -1

                    if(template is TemplateTouTiaoSdkRewardVideo){
                        jsonObject.put("uniqueId",template.uniqueid)
                        mOpenType = 1
                    }

                    if(template is TemplateGdtSdkRewardVideo){
                        jsonObject.put("uniqueId",template.uniqueid);
                        mOpenType = 1
                    }

                    val itemBase = template.defaultItem
                    if(itemBase is ApullMvItem){
                        jsonObject.put("uniqueId",itemBase.unique_id)
                        when{
                            itemBase.interaction_type == 3 -> mOpenType = 1
                            itemBase.interaction_type == 2 -> mOpenType = 2
                            !itemBase.interaction_object?.deeplink.isNullOrEmpty() -> mOpenType = 3
                        }

                        if(itemBase.package_name.isNullOrEmpty().not()
                                && PackageUtil.isPkgInstalled(context,itemBase.package_name)){
                            if(itemBase.filter_type == 0){
                                itemBase.status = ApullConst.STATUS_TYPE_INSTALL_FINISHED
                            }
                        }
                        if(itemBase.status == ApullConst.STATUS_TYPE_INSTALL_FINISHED &&
                                itemBase.interaction_object?.deeplink.isNullOrEmpty().not()){
                            mOpenType = 3
                        }
                    }

                    if(itemBase is ApullTouTiaoItem){
                        jsonObject.put("uniqueId",itemBase.unique_id)
                        when{
                            itemBase.interaction_type == 4 -> mOpenType = 1
                            itemBase.interaction_type == 3 || itemBase.iType ==2 -> mOpenType = 2
                        }
                    }
                    jsonObject.put("openType",mOpenType)
                }
            }
            Logger.d("ApullRewardVideoInterface",jsonObject.toString())
            return jsonObject.toString()
        }
    }

}