package com.qihoo360.mobilesafe.reward.container.gdt

import android.content.Context
import android.text.TextUtils
import android.util.AttributeSet
import android.view.View
import android.widget.Toast

import com.qihoo360.apullsdk.apull.ApullConst
import com.qihoo360.apullsdk.apull.control.sync.DownloadSatusManager
import com.qihoo360.apullsdk.apull.protocol.ApullReportManager
import com.qihoo360.apullsdk.apull.protocol.model.ApullTemplateBase
import com.qihoo360.apullsdk.apull.protocol.model.impl.TemplateGdt
import com.qihoo360.apullsdk.apull.protocol.model.impl.gdt.ApullGdtItem
import com.qihoo360.apullsdk.apull.protocol.network.report.ReportNetworkGDT
import com.qihoo360.apullsdk.apull.view.impl.ContainerGdtBase
import com.qihoo360.apullsdk.utils.NetUtil

import com.qihoo360.mobilesafe.reward.R
import com.qihoo360.mobilesafe.reward.container.displayImage
import com.qihoo360.mobilesafe.reward.data.ApullItemWrapper
import com.qihoo360.mobilesafe.reward.data.RewardRecordManager
import com.qihoo360.mobilesafe.reward.presenter.view.RewardView
import com.qihoo360.mobilesafe.reward.ui.page.dialog.RewardDownloadDialog
import com.qihoo360.mobilesafe.support.log.Logger
import com.qihoo360.mobilesafe.ui.common.dialog.CommonDialog
import kotlinx.android.synthetic.main.apullsdk_container_apull_gdt_2105.view.*

class ContainerGdt2105 : ContainerGdtBase {

    companion object {

        const val TAG = "ContainerGdt2105"
    }

    constructor(context: Context, attrs: AttributeSet) :super(context,attrs)

    constructor(context: Context,template: ApullTemplateBase?) :super(context,template)

    override fun initView(template: ApullTemplateBase?) {
        View.inflate(context, R.layout.apullsdk_container_apull_gdt_2105, this)
        initDownloadButton(mRewardBtn2105)
        // 注册下载状态监控
        DownloadSatusManager.register(this)
    }

    override fun updateView(template: ApullTemplateBase?) {
       if(template is TemplateGdt && template != mTemplateGdt){
           mTemplateGdt = template
           mGdtItem = mTemplateGdt.defaultItem
           debugColor()
           updateText()
           updateImage()
           addClickListener()
       }
    }

    private fun debugColor() {
        if(Logger.bDebug()){
            this.setBackgroundColor(resources.getColor(R.color.progress_white))
        }
    }

    private fun addClickListener() {
        initRootClick(mRoot2105)
    }

    override fun handleRootClick(clickParams: ApullGdtItem.GdtClickParams?) {
        handleDownLoadBtnClick(clickParams)
    }

    private fun updateImage() {
        mLargeImage2105?.apply {
            val url = mGdtItem?.img2_url?:mGdtItem?.img_url
            displayImage(url)
        }
    }

    private fun updateText() {
        mAppName2105?.text = mGdtItem?.title
        if(mGdtItem?.status == ApullConst.STATUS_TYPE_INSTALL_FINISHED){
            mRewardBtn2105?.text = context.getText(R.string.open)
        }else{
            mRewardBtn2105?.text = context.getText(R.string.get_hongbao)
        }

    }

    override fun onImageEnableChange(enable: Boolean) {
        updateImage()
    }

    override fun onThemeChanged() {
    }

    override fun updateDownloadStatus() {
        post{
            updateText()
        }
    }

    override fun startDownloadApp(type: CLICKTYPE?, clickParams: ApullGdtItem.GdtClickParams?) {
        if(context is RewardView){
            (context as? RewardView)?.showRewardDownloadDialog(ApullItemWrapper(mTemplateGdt), mDownloadListener)
        }
        RewardRecordManager.setLastRewardUniqueId(context,mGdtItem?.unique_id)
        super.startDownloadApp(type, clickParams)
    }

    override fun openApp(isAuto: Boolean) {
        RewardRecordManager.setLastRewardUniqueId(context,mGdtItem?.unique_id)
        super.openApp(isAuto)
    }

    fun startDownloadAppNoDialog(type: CLICKTYPE?, clickParams: ApullGdtItem.GdtClickParams?) {
        super.startDownloadApp(type,clickParams)
    }

    override fun startDownloadAppWithTips( type :CLICKTYPE,clickParams: ApullGdtItem.GdtClickParams) {
        if (NetUtil.isConnected(context)) {
            if (!NetUtil.isWifiConnected(context)) {
                val title = context.getString(R.string.tips_title)
                val body = context.getString(R.string.tips_body_start_download)
                val ok = context.getString(R.string.ok)
                val cancel = context.getString(R.string.cancel)
                val dialog = CommonDialog(context,title,body)
                dialog.setBtnOk(ok) {
                    startDownloadApp(type,clickParams)
                    dialog.dismiss()
                }
                dialog.setBtnCancel(cancel){
                    dialog.dismiss()
                }
                dialog.show()

            } else {
                startDownloadApp(type,clickParams)
            }
        } else {
            Toast.makeText(context, R.string.hongbao_net_no_connect_tips, Toast.LENGTH_SHORT).show()
        }
    }

    private val mDownloadListener = object : RewardDownloadDialog.DownloadListener {

        override fun onClick() {
            if (mGdtItem.status == ApullConst.STATUS_TYPE_DOWNLOAD_PAUSED) {
                startDownloadAppNoDialog(CLICKTYPE.TYPE_BUTTON, clickParams)
                return
            }
            handleDownLoadBtnClick(clickParams)
        }

        override fun onDismiss() {
            if (mGdtItem.status == ApullConst.STATUS_TYPE_DOWNLOADING ||
                    mGdtItem.status == ApullConst.STATUS_TYPE_UPDATE_PROGRESS) {
                pauseDownloadApp()
            }
        }

        override fun onContinue() {
            if (mGdtItem.status == ApullConst.STATUS_TYPE_DOWNLOAD_PAUSED) {
                startDownloadAppNoDialog(CLICKTYPE.TYPE_BUTTON, clickParams)
            }
        }

    }

    override fun onApkInstalled(downloadid: String, iType: Int) {
        if (!TextUtils.isEmpty(downloadid) && downloadid == mTemplateGdt.downloadid) {
            mGdtItem.status = ApullConst.STATUS_TYPE_INSTALL_FINISHED
            if (!mGdtItem.installed_reported) {
                mGdtItem.installed_reported = true
                ApullReportManager.reportApullGDT(mTemplateGdt, ReportNetworkGDT.REPORTTYPE.INSTALLFINISH)
                ApullReportManager.reportApullSspGdtInstalled(context, mTemplateGdt)
            }
            updateDownloadStatus()

            Logger.d(TAG, "onApkInstalled downloadid:$downloadid")
        }
    }


}