package com.qihoo360.mobilesafe.reward.presenter

import android.os.Handler
import android.os.Message
import com.qihoo360.mobilesafe.reward.data.RewardUtils
import com.qihoo360.mobilesafe.reward.respository.RewardCashListResponse
import com.qihoo360.mobilesafe.reward.respository.RewardCommitResponse
import com.qihoo360.mobilesafe.reward.respository.RewardTryResponse
import com.qihoo360.mobilesafe.reward.ui.page.dialog.RewardDownloadDialog
import com.qihoo360.mobilesafe.reward.ui.page.dialog.RewardDownloadDialog.Companion.DONT_HAS_CHANCE
import com.qihoo360.newssdk.protocol.model.TemplateBase
import java.lang.ref.WeakReference

class RewardMainHandler(outer:RewardPresenter) : Handler(){

     private val mOuter: WeakReference<RewardPresenter> = WeakReference(outer)

    override fun handleMessage(msg: Message?) {
        val rewardPresenter = mOuter.get() ?: return
        if(msg == null)
            return
        when(msg.what){
            MSG_ON_RESPONSE_TIMEOUT -> {
                rewardPresenter.handleRewardTimeout()
                removeMessages(MSG_ON_RESPONSE_TIMEOUT)
            }

            MSG_ON_RESPONSE_SUCCESS ->{
                if(hasMessages(MSG_ON_RESPONSE_TIMEOUT)){
                    removeMessages(MSG_ON_RESPONSE_TIMEOUT)
                    if(msg.obj is MutableList<*>?){
                        rewardPresenter.handleRewardSuccess(msg.obj as? MutableList<TemplateBase>?,msg.arg1,msg.arg2)
                    }else{
                        rewardPresenter.handleRewardEmpty()
                    }
                }
            }

            MSG_ON_TRY_TIMEOUT -> {
                if(hasMessages(MSG_ON_TRY_RESPONSE)) {
                    removeMessages(MSG_ON_TRY_RESPONSE)
                }
                val rewardInfo = msg.obj as? RewardUtils.RewardInfo
                rewardPresenter.handleTryTimeout(rewardInfo)
            }

            MSG_ON_TRY_RESPONSE -> {
                val rewardInfo = msg.obj as? RewardUtils.RewardInfo
                rewardPresenter.handleTryResult(rewardInfo)
            }

            MSG_ON_COMMIT_TIMEOUT -> {
                if(hasMessages(MSG_ON_COMMIT_RESPONSE)){
                    removeMessages(MSG_ON_COMMIT_RESPONSE)
                }
                val rewardInfo = msg.obj as? RewardUtils.RewardInfo
                rewardPresenter.handleCommitTimeout(rewardInfo)
            }

            MSG_ON_COMMIT_RESPONSE ->{
                val rewardInfo = msg.obj as? RewardUtils.RewardInfo
                rewardPresenter.handleCommitResult(rewardInfo)
            }

            MSG_ON_TOAST_SUCCESS ->{
                rewardPresenter.handleToastSuccess()
            }

            MSG_ON_TOAST_SUCCESS_HAS_MONEY ->{
                rewardPresenter.handleToastSuccessHasMoney(msg.obj as Int)
            }

            MSG_ON_DEEP_LINK_BACK ->{
                val rewardInfo = msg.obj as? RewardUtils.RewardInfo
                rewardPresenter.handleDeepLinkResult(rewardInfo)
            }

            MSG_ON_TOAST_FAIL ->{
                val rewardInfo = msg.obj as? RewardUtils.RewardInfo
                rewardPresenter.handleNoRewardChance(rewardInfo,DONT_HAS_CHANCE,rewardInfo?.rewardType ?:RewardUtils.REWARD_TYPE_APP)
            }

            MSG_ON_CASH_LIST_TIME_OUT ->{
                if(hasMessages(MSG_ON_CASH_LIST_RESPONSE)){
                    removeMessages(MSG_ON_CASH_LIST_RESPONSE)
                }

                rewardPresenter.handleRewardCashListTimeOut()
            }

            MSG_ON_CASH_LIST_RESPONSE ->{
                if(hasMessages(MSG_ON_CASH_LIST_TIME_OUT)){
                    removeMessages(MSG_ON_CASH_LIST_TIME_OUT)
                }
                rewardPresenter.handleRewardCashList(msg.obj as? RewardCashListResponse)
            }

        }
    }

    companion object {

        const val MSG_ON_RESPONSE_TIMEOUT = 0

        const val MSG_ON_RESPONSE_SUCCESS = 1

        const val MSG_ON_RESPONSE_EMPTY = 2

        const val MSG_ON_TRY_RESPONSE = 3

        const val MSG_ON_COMMIT_RESPONSE = 4

        const val MSG_ON_TRY_TIMEOUT = 5

        const val MSG_ON_COMMIT_TIMEOUT = 6

        const val MSG_ON_TOAST_SUCCESS = 7

        const val MSG_ON_TOAST_SUCCESS_HAS_MONEY = 8

        const val MSG_ON_TOAST_FAIL = 9

        const val MSG_ON_CAN_TRY = 10

        const val MSG_ON_DEEP_LINK_TOAST = 11

        const val MSG_ON_DEEP_LINK_BACK = 12//deeplink需要返回3s后可以拆

        const val MSG_ON_CASH_LIST_RESPONSE = 13

        const val MSG_ON_CASH_LIST_TIME_OUT = 14

        const val MSG_REWARD_TIMEOUT:Long = 8000

        const val MSG_TRY_COMMIT_TIMEOUT:Long = 5000
    }

    fun postRewardRequestTimeOut(){
        sendEmptyMessageDelayed(MSG_ON_RESPONSE_TIMEOUT,MSG_REWARD_TIMEOUT)
    }

    fun postRewardResponseSuccess(list: MutableList<TemplateBase>?,scene:Int,subScene:Int){
        list?.apply {
            val msg = obtainMessage()
            msg.what = MSG_ON_RESPONSE_SUCCESS
            msg.obj = list
            msg.arg1 = scene
            msg.arg2 = subScene
            sendMessage(msg)
        }?:kotlin.run {
            postRewardResponseEmpty()
        }
    }

    private fun postRewardResponseEmpty(){
        sendEmptyMessage(MSG_ON_RESPONSE_EMPTY)
    }

    fun postRewardTryTimeout(rewardInfo: RewardUtils.RewardInfo){
        val msg = obtainMessage()
        msg.what = MSG_ON_TRY_TIMEOUT
        msg.obj = rewardInfo
        sendMessageDelayed(msg,MSG_TRY_COMMIT_TIMEOUT)
    }

    fun postRewardTrySuccess(response: RewardTryResponse?,rewardInfo: RewardUtils.RewardInfo){
        rewardInfo.tryResponse = response
        if(hasMessages(MSG_ON_TRY_TIMEOUT)) {
            removeMessages(MSG_ON_TRY_TIMEOUT)
        }
        val msg = obtainMessage()
        msg.what = MSG_ON_TRY_RESPONSE
        msg.obj = rewardInfo
        sendMessage(msg)
    }

    fun postRewardCashList(response:RewardCashListResponse?){
        if(hasMessages(MSG_ON_CASH_LIST_TIME_OUT)){
            removeMessages(MSG_ON_CASH_LIST_TIME_OUT)
        }
        val msg = obtainMessage()
        msg.what = MSG_ON_CASH_LIST_RESPONSE
        msg.obj = response
        sendMessage(msg)
    }

    fun postRewardCashListTimeOut(){
        val msg = obtainMessage()
        msg.what = MSG_ON_CASH_LIST_TIME_OUT
        sendMessageDelayed(msg,MSG_TRY_COMMIT_TIMEOUT)
    }

    fun postRewardCommitTimeout(rewardInfo: RewardUtils.RewardInfo){
        if(hasMessages(MSG_ON_COMMIT_RESPONSE)) {
            removeMessages(MSG_ON_COMMIT_RESPONSE)
        }
        if(hasMessages(MSG_ON_TOAST_SUCCESS)){
            removeMessages(MSG_ON_TOAST_SUCCESS)
        }
        val msg = obtainMessage()
        msg.what = MSG_ON_COMMIT_TIMEOUT
        msg.obj = rewardInfo
        sendMessageDelayed(msg,MSG_TRY_COMMIT_TIMEOUT)
    }

    fun postRewardCommitSuccess(response: RewardCommitResponse?,rewardInfo: RewardUtils.RewardInfo){
        rewardInfo.commitResponse = response
        if(hasMessages(MSG_ON_COMMIT_TIMEOUT)) {
            removeMessages(MSG_ON_COMMIT_TIMEOUT)
        }
        val msg = obtainMessage()
        msg.what = MSG_ON_COMMIT_RESPONSE
        msg.obj = rewardInfo
        sendMessage(msg)
    }

    fun postRewardToastSuccess(){
        sendEmptyMessageDelayed(MSG_ON_TOAST_SUCCESS, MSG_TRY_COMMIT_TIMEOUT)
    }

    fun postRewardToastSuccessHasMoney(amount :Int){
        if(hasMessages(MSG_ON_TOAST_SUCCESS) && amount > 0){
            removeMessages(MSG_ON_TOAST_SUCCESS)
            obtainMessage(MSG_ON_TOAST_SUCCESS_HAS_MONEY,amount).sendToTarget()
        }
    }

    fun postRewardDeepLinkType(rewardInfo: RewardUtils.RewardInfo?){
        rewardInfo?.apply {
            val msg = obtainMessage()
            msg.what = MSG_ON_DEEP_LINK_BACK
            msg.obj = rewardInfo
            sendMessageDelayed(msg,3000)
        }
    }

    fun postNoRewardChance(rewardInfo: RewardUtils.RewardInfo?,timeDelay:Long){
        val msg = obtainMessage()
        msg.what = MSG_ON_TOAST_FAIL
        msg.obj = rewardInfo
        sendMessageDelayed(msg,timeDelay)
    }
}